<?php
/**
 * @package	JoomSMS for Joomla!
 * @version	3.1.0
 * @author	joomboost.com
 * @copyright	(C) 2013-2020 JoomBoost. All rights reserved.
 * @license	GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php

class JOOMSMSGateway_mvaayoo_gateway extends JOOMSMSGateway_default_gateway{

	public $username;
	public $password;
	public $senderid;
	public $waittosend = 0;

	public $sendMessage = true;
	public $deliveryReport = false;
	public $answerManagement = false;

	public $domain = "api.mVaayoo.com";
	public $port = 80;

	public $name = 'mVaayoo';
	public $creditsUrl = 'http://www.mvaayoo.com/pricing.html';

	public function openSend($message, $phone){

		$params = array();
		$params['receipientno'] = $this->checkNum($phone);
		$params['user'] = $this->username.':'.$this->password;
		$params['msgtxt'] = $message;
		$params['state'] = 1;
		$params['senderID'] = $this->senderid;


		$stringToPost = '';
		foreach($params as $oneParam => $value){
			$value = urlencode(($value));
			$stringToPost .= '&'.$oneParam.'='.$value;
		}

		$stringToPost = ltrim($stringToPost, '&');

		$fsockParameter = "POST /mvaayooapi/MessageCompose HTTP/1.1\r\n";
		$fsockParameter .= "Host: api.mVaayoo.com\r\n";
		$fsockParameter .= "Content-Type: application/x-www-form-urlencoded\r\n";
		$fsockParameter .= "Content-length: ".strlen($stringToPost)."\r\n\r\n";
		$fsockParameter .= $stringToPost;

		return $this->sendRequest($fsockParameter);
	}

	public function displayConfig(){
		?>
		<table>
			<tr>
				<td>
					<label for="senderprofile_senderid"><?php echo JText::_('SMS_SENDER_ID'); ?></label>
				</td>
				<td>
					<input type="text" name="data[senderprofile][senderprofile_params][senderid]" id="senderprofile_senderid" class="inputbox" style="width:200px;" value="<?php echo htmlspecialchars(@$this->senderid, ENT_COMPAT, 'UTF-8'); ?>"/>
				</td>
			</tr>
			<tr>
				<td>
					<label for="senderprofile_username"><?php echo JText::_('SMS_USERNAME'); ?></label>
				</td>
				<td>
					<input type="text" name="data[senderprofile][senderprofile_params][username]" id="senderprofile_username" class="inputbox" style="width:200px;" value="<?php echo htmlspecialchars(@$this->username, ENT_COMPAT, 'UTF-8'); ?>"/>
				</td>
			</tr>
			<tr>
				<td>
					<label for="senderprofile_password"><?php echo JText::_('SMS_PASSWORD') ?></label>
				</td>
				<td>
					<input name="data[senderprofile][senderprofile_params][password]" id="senderprofile_password" class="inputbox" type="password" style="width:200px;" value="<?php echo htmlspecialchars(@$this->password, ENT_COMPAT, 'UTF-8'); ?>"/>
				</td>
			</tr>
			<tr>
				<td colspan="2">
					<label for="senderprofile_waittosend"><?php echo JText::sprintf('SMS_WAIT_TO_SEND', '<input type="text" style="width:20px;" name="data[senderprofile][senderprofile_params][waittosend]" id="senderprofile_waittosend" class="inputbox" value="'.intval($this->waittosend).'" />'); ?></label>
				</td>
			</tr>
		</table>

		<?php
	}

	public function afterSaveConfig($senderprofile){
		if(in_array(JRequest::getCmd('task'), array('save', 'apply'))) $this->displayBalance();
	}

	public function getBalance(){
		$fsockParameter = "GET /mvaayooapi/APIUtil?user=".$this->username.":".$this->password."&type=0 HTTP/1.1\r\n";
		$fsockParameter .= "Host: api.mvaayoo.com \r\n\r\n";

		$idConnection = $this->sendRequest($fsockParameter);
		$result = $this->readResult($idConnection);

		if(!strpos($result, '200 OK')){
			$this->errors[] = 'Error 200 KO => '.$result;
			return false;
		}else $res = trim(substr($result, strpos($result, "\r\n\r\n")));

		return array("default" => $res);
	}

	private function displayBalance(){
		$balance = $this->getBalance();
		if($balance === false){
			JOOMSMS::enqueueMessage(implode('<br />', $this->errors), 'error');
			return false;
		}
		JOOMSMS::enqueueMessage($balance["default"]);
	}

	protected function interpretSendResult($result){
		if(!strpos($result, '200 OK')){
			$this->errors[] = 'Error 200 KO => '.$result;
			return false;
		}else $res = trim(substr($result, strpos($result, "\r\n\r\n")));
		if(strpos($res, 'Status=1') === false) return true;
		$this->errors[] = $res;
		return false;
	}

	protected function checkNum($phone){
		if(strpos($phone, '+91') === false){
			$this->errors[] = 'The phone number is not a valid Singapore phone or a Canadian phone number';
			return false;
		}
		$singaporePhone = str_replace('+91', '', $phone);
		return $singaporePhone;
	}
}
