<?php
/**
 * @package        JoomSMS
 * @copyright      2013-2020 JoomBoost, joomboost.com
 * @license        GNU/GPL http://www.gnu.org/copyleft/gpl.html
 */

// No direct access
defined('_JEXEC') or die;

JLoader::register('JoomsmsHelperStats',JPATH_ADMINISTRATOR.'/components/com_joomsms/helpers/stats.php');

JLoader::register('JoomsmsHelperVersion',JPATH_ADMINISTRATOR.'/components/com_joomsms/helpers/version.php');

class JoomsmsHelperDashboard
{

	static public function getLeftWidgets(){

		//inits
		$leftWidgets = [];
		$recently = JoomsmsHelperStats::getRecently();

		// recent bookings graph
		$leftWidgets[] = [
			'title'       => 'COM_JOOMSMS_GRAPH_RECENTLY',
			'icon'        => 'chart-line',
			'bodyContent' => JLayoutHelper::render('dashboard.recently', ['recent' => $recently])
		];


		if(JFactory::getUser()->authorise('core.admin'))
		{
			// only super user can see update widget
			// left widgets
			$leftWidgets[] = [
				'title'       => 'COM_JOOMSMS_CONFIGURATION',
				'icon'        => 'list',
				'bodyContent' => JLayoutHelper::render('dashboard.config'),
			];

			$leftWidgets[] = [
				'title'       => 'COM_JOOMSMS_PRODUCT_INFO',
				'icon'        => 'info',
				'bodyContent' => JLayoutHelper::render('dashboard.productInfo'),
			];

			$leftWidgets[] = [
				'title'       => 'COM_JOOMSMS_FOLLOWUS',
				'icon'        => 'share',
				'bodyContent' => JLayoutHelper::render('dashboard.socialLinks'),
			];

		}

		$leftWidgets[] = [
			'bodyContent' => JLayoutHelper::render('dashboard.copyright'),
		];


		return $leftWidgets;
	}


	static public function getRightWidgets(){

		// inits
		$rightWidgets = [];

		if(JFactory::getUser()->authorise('core.admin')){ // only super user can see update widget

			$rightWidgets[] = [
				'title'         => 'COM_JOOMSMS_DB_UPDATES',
				'icon'          => 'sync',
				'bodyContent'   => JLayoutHelper::render('dashboard.downloadId'),
				'footerContent' => JLayoutHelper::render('dashboard.downloadIdFooter')
			];

		}



		if(JFactory::getUser()->authorise('core.admin')){ // only super user can see joomboost company social widgets

			$rightWidgets[] = [
				'bodyContent' => JLayoutHelper::render('dashboard.facebookPage')
			];

			$rightWidgets[] = [
				'bodyContent' => JLayoutHelper::render('dashboard.twitterFollow')
			];

		}

		return $rightWidgets;

	}


	static public function getCountCards(){

        // Recivers
        $buttons[] = array(

            'link'       => 'index.php?option=com_joomsms&ctrl=receiver',
            'iconName'   => 'fas fa-user-friends',
            'count'      => JoomsmsHelperStats::getCount(null,'joomsms_user'),
            'iconClass'  => 'bg-info text-white',
            'countClass' => 'text-info',
            'title'      => 'COM_JOOMSMS_DASH_RECEIVERS'
        );


        // Messages
        $buttons[] = array(

            'link'       => 'index.php?option=com_joomsms&ctrl=message',
            'iconName'   => 'fas fa-comments',
            'count'      => JoomsmsHelperStats::getCount(null,'joomsms_message'),
            'iconClass'  => 'bg-warning text-white',
            'countClass' => 'text-warning',
            'title'      => 'COM_JOOMSMS_DASH_MESSAGES'
        );

        // Answers
        $buttons[] = array(

            'link'       => 'index.php?option=com_joomsms&ctrl=answer',
            'iconName'   => 'fas fa-comment',
            'count'      => JoomsmsHelperStats::getCount(null,'joomsms_answer'),
            'iconClass'  => 'bg-danger text-white',
            'countClass' => 'text-danger',
            'title'      => 'COM_JOOMSMS_DASH_ANSWERS'
        );

        // Customers
        $buttons[] = array(

            'link'       => 'index.php?option=com_joomsms&ctrl=customer',
            'iconName'   => 'fas fa-users',
            'count'      => JoomsmsHelperStats::getCount(null,'joomsms_customer'),
            'iconClass'  => 'bg-primary text-white',
            'countClass' => 'text-primary',
            'title'      => 'COM_JOOMSMS_DASH_CUSTOMERS'
        );


        return $buttons;
	}


	static public function getSkeleton(){
		return JLayoutHelper::render(
			'dashboard.skeleton.container',
			[
				'counts' => self::getCountCards(),
				'left'   => self::getLeftWidgets(),
				'right'  => self::getRightWidgets()
			]
		);
	}


}