<?php
/**
 * @package	JoomSMS for Joomla!
 * @version	1.0
 * @author	joomboost.com
 * @copyright	(C) 2013-2020 JoomBoost. All rights reserved.
 * @license	GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');

class JoomsmsControllerVerification extends JControllerForm{

    public $data = [];
    /*
     * phone number validated and corrected automatically
     */
    private $phoneNumber = null;
    /*
     * original phone in joomla user
     */
    private $phoneNumberOriginal = null;
    /**
     * @var    \Joomla\CMS\User\User  The user registration data.
     * @since  1.6
     */
    private $user = null;
    private $newPassword = null;


    public function resetPassword(){

        // check token
        $this->checkToken();

        // validate phone number;
        $this->validatePhoneNumber();


        // check if phone number is an already registered user
        $this->checkJoomlaUserPhone();

        // reset password
        $this->resetJoomlaUserPassword();


        // send message over SMS
        $this->sendPasswordOverSMS();

        // all success
        die();



    }

    private function resetJoomlaUserPassword(){

        // set the new password
        $this->user->password_clear = JUserHelper::genRandomPassword();
        $this->user->password = JApplicationHelper::getHash($this->user->password_clear);

        // save the new password
        $result = $this->user->save(true);

        if(!$result){
            // validation error
            $this->data['message'] = JText::_('Password Reset Error');
            $this->data['error'] = 'true';

            $this->ajax();

            return false;
        }

    }

    private function sendPasswordOverSMS(){
        // some inits

        $senderProfileClass = JOOMSMS::get('class.senderprofile');

        $gateway = $senderProfileClass->getGateway();
        if(!$gateway->open()){
            // validation error
            $this->data['message'] =  JText::_('Gateway Error');
            $this->data['error'] = 'true';

            $this->ajax();

            return false;
        }

        $status = $gateway->send(JText::_('COM_JOOMSMS_YOUR_NEW_PASSWORD').' '.$this->user->password_clear,$this->phoneNumber);
        $errors = $gateway->errors;
        $gateway->close();

        if(!$status){
            // validation error
            $this->data['message'] =  JText::sprintf('SMS_ERROR_SENT', '', '<b><i>'.$this->phoneNumberOriginal.'</i></b>').'<br />'.implode('<br />', $errors);
            $this->data['error'] = 'true';

            $this->ajax();

            return false;
        }


        // validation error
        $this->data['message'] =  JText::_('COM_JOOMSMS_SUCC_SENT_TO').' <b><i>'.$this->phoneNumberOriginal.'</i></b>';
        $this->data['error'] = 'false';

        $this->ajax();

        return true;

    }

    private function checkJoomlaUserPhone(){

        $db = JFactory::getDbo();

        $phonesNumber = [$db->quote('"'.$this->phoneNumber.'"'),$db->quote('"'.$this->phoneNumberOriginal.'"')];
        $phonesNumber = implode(',',$phonesNumber);


        $query = $db->getQuery(true);
        $query->select('u.*');
        $query->from('#__users as u');
        $query->leftJoin('#__user_profiles up on up.user_id = u.id');
        $query->where("up.profile_key = 'profile.phone'");
        $query->where("(up.profile_value IN ($phonesNumber))");


        $db->setQuery($query);

        $result = $db->loadObject();

        if(is_null($result)){

            // validation error
            $this->data['message'] = JText::_('COM_JOOMSMS_USER_NOT_FOUND_WITH_THIS_PHONE');
            $this->data['error'] = 'true';

            $this->ajax();

            return false;

        }

        // load user
        $this->user = JFactory::getUser($result->id);

        return true;

    }

    private function validatePhoneNumber(){


        // get phone number
        $this->phoneNumber = $this->phoneNumberOriginal = \Joomla\CMS\Factory::getApplication()->input->get('phone','','string');

        // load phone class helper
        $phoneHelper = JOOMSMS::get('helper.phone');

        // validate phone number
        $this->phoneNumber = $phoneHelper->getValidNum($this->phoneNumber);

        // validation success
        if($this->phoneNumber)
            return true;

        // validation error
        $this->data['message'] = $phoneHelper->error;
        $this->data['error'] = 'true';

        $this->ajax();


        return false;

    }



    public function ajax(){

        header('Content-Type: application/json; charset=utf-8');

        echo json_encode($this->data);

        die();

    }


    public function checkToken($method = 'post', $redirect = true)
    {

        $token = parent::checkToken('get', false);

        // check token first
        if(!$token){
            $this->data['message'] = 'invalid token';
            $this->data['error'] = 'true';

            $this->ajax();
        }


        return true;



    }


}