<?php
/**
 * @package	JoomSMS for Joomla!
 * @version	1.0
 * @author	JoomBoost
 * @copyright	(C) 2013-2020 JoomBoost. All rights reserved.
 * @license	GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');


class plgSystemJoomsmsUserCreation extends JPlugin{


	function __construct(&$subject, $config){
		if(!file_exists(rtrim(JPATH_ADMINISTRATOR, DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_joomsms'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'helper.php')) return false;
		parent::__construct($subject, $config);
	}

	private function init(){
		if(defined('JOOMSMS_COMPONENT')) return true;
		$joomSmsHelper = rtrim(JPATH_ADMINISTRATOR, DIRECTORY_SEPARATOR).DIRECTORY_SEPARATOR.'components'.DIRECTORY_SEPARATOR.'com_joomsms'.DIRECTORY_SEPARATOR.'helpers'.DIRECTORY_SEPARATOR.'helper.php';
		if(file_exists($joomSmsHelper)) include_once $joomSmsHelper;

		return defined('JOOMSMS_COMPONENT');
	}

	function onJOOMSMSGetMessageType(&$types, $integration){
		if(!$this->init()) return;

		if($integration == 'acymailing' || $integration == 'joomsms' || empty($integration)) return;

		$newType = new stdClass();
		$newType->name = JText::_('SMS_AUTO_USER_CREATION');
		$types['usercreation'] = $newType;
		return;
	}

	function onJOOMSMSDisplayParamsAutoMessage_usercreation(){
		$timevalue = array();
		$timevalue[] = JHTML::_('select.option', 'hours', JText::_('SMS_HOURS'));
		$timevalue[] = JHTML::_('select.option', 'days', JText::_('SMS_DAYS'));
		$timevalue[] = JHTML::_('select.option', 'weeks', JText::_('SMS_WEEKS'));
		$timevalue[] = JHTML::_('select.option', 'months', JText::_('SMS_MONTHS'));
		$delay = JHTML::_('select.genericlist', $timevalue, "data[message][message_receiver][auto][usercreation][delay][timevalue]", 'size="1"  style="width:auto;"', 'value', 'text', '0');

		$sentTrigger = array();
		$sentTrigger[] = JHTML::_('select.option', 'creation', JText::_('SMS_USER_CREATED'));
		$sentTrigger[] = JHTML::_('select.option', 'confirmation', JText::_('SMS_USER_CONFIRMED'));
		$sentOption = JHTML::_('select.genericlist', $sentTrigger, "data[message][message_receiver][auto][usercreation][sentTrigger]", 'size="1" style="width:auto;"', 'value', 'text', '0');

		$timeNumber = '<input type="text" name="data[message][message_receiver][auto][usercreation][delay][duration]" class="inputbox" style="width:30px" value="0">';
		echo JText::sprintf('SMS_SEND_CREATION_AUTO', $timeNumber.' '.$delay, $sentOption).'<br />';
		echo '<div id="loadField"></div>';
	}

	function onUserAfterSave($user, $isnew, $success, $msg){
		if(!$this->init()) return;

		$config = JOOMSMS::config();
		$allowCustomerManagement = $config->get('allowCustomersManagement', '0');
		if($allowCustomerManagement && $isnew) $this->_createCustomer($user);

		return $this->onAfterStoreUser($user, $isnew, $success, $msg);
	}

	function onUserBeforeSave($user, $isnew, $new){
		return $this->onBeforeStoreUser($user, $isnew);
	}


	function onBeforeStoreUser($user, $isnew){

		if(is_object($user)) $user = get_object_vars($user);

		$this->oldUser = $user;

		return true;
	}

	function onAfterStoreUser($user, $isnew, $success, $msg){
		$mainframe = JFactory::getApplication();

		if(is_object($user)) $user = get_object_vars($user);

		if($success === false) return false;
		if(empty($user['id'])) return false;

		$mainframe->setUserState("JOOMSMS_userCreationId", intval($user['id']));
		if(!empty($user['password_clear'])) $mainframe->setUserState("JOOMSMS_userPwd", base64_encode($user['password_clear']));

		if($isnew){
			$mainframe->setUserState("JOOMSMS_action", 'creation');
		}else{            //Confirm the user if the status changed so we moved from blocked to non blocked
			if(!empty($this->oldUser['block']) && empty($user['block'])) $mainframe->setUserState("JOOMSMS_action", 'confirmation');
		}

		return true;
	}

	public function onAfterDispatch(){
		if(!$this->init()) return;

		$db = JFactory::getDBO();
		$mainframe = JFactory::getApplication();

		$userCreationId = $mainframe->getUserStateFromRequest("JOOMSMS_userCreationId", "JOOMSMS_userCreationId", "0");
		if(empty($userCreationId)) return;

		$messageClass = JOOMSMS::get('class.message');
		$allMessages = $messageClass->getAutoMessage('usercreation');
		if(empty($allMessages)) return false;

		$paramqueue = '';
		$userPwd = $mainframe->getUserStateFromRequest("JOOMSMS_userPwd", "JOOMSMS_userPwd", "0");
		if(!empty($userPwd)){
			$params = new stdClass();
			$params->password = $userPwd;
			$paramqueue = serialize($params);
		}

		$sentTrigger = $mainframe->getUserStateFromRequest("JOOMSMS_action", "JOOMSMS_action", "0");

		$mainframe->setUserState("JOOMSMS_userCreationId", 0);
		$mainframe->setUserState("JOOMSMS_userPwd", 0);
		$mainframe->setUserState("JOOMSMS_action", 0);

		foreach($allMessages as $oneMessage){
			if(empty($sentTrigger) || $sentTrigger != $oneMessage->message_receiver['auto']['usercreation']['sentTrigger']) continue;

			$senddate = strtotime('+'.intval($oneMessage->message_receiver['auto']['usercreation']['delay']['duration']).' '.$oneMessage->message_receiver['auto']['usercreation']['delay']['timevalue'], time());

			$integration = JOOMSMS::getIntegration($oneMessage->message_receiver_table);
			$receiverId = $integration->getReceiverIDs($userCreationId);


			$joomsmsquery = JOOMSMS::get('class.joomsmsquery');
			$integrationFrom = $oneMessage->message_receiver_table;
			$integrationTo = $oneMessage->message_receiver_table;
			$integration = JOOMSMS::getIntegration($integrationTo);
			$integration->initQuery($joomsmsquery);
			$joomsmsquery->addMessageFilters($oneMessage);
			$joomsmsquery->addUserFilters($receiverId, $integrationFrom, $integrationTo);
			$querySelect = $joomsmsquery->getQuery(array($oneMessage->message_id.','.$integration->tableAlias.'.'.$integration->primaryField.','.$db->Quote($oneMessage->message_receiver_table).','.$senddate.',0,2,'.$db->Quote($paramqueue)));

			$finalQuery = 'INSERT IGNORE INTO `#__joomsms_queue` (`queue_message_id`,`queue_receiver_id`,`queue_receiver_table`,`queue_senddate`,`queue_try`,`queue_priority`, `queue_paramqueue`) '.$querySelect;
			$db->setQuery($finalQuery);
			$db->query();
		}
	}




	public function onJoomSMSSaveMessage($oldMsg, $newMsg){
		if(!$this->init()) return;

		if(empty($newMsg['message_type']) || $newMsg['message_type'] != 'auto' || empty($newMsg['message_autotype'])) return;
		if($newMsg['message_autotype'] != 'usercreation') return;
		$toggleHelper = JOOMSMS::get('helper.toggle');

		if(!isset($newMsg['message_receiver']['auto']['usercreation']['delay'])) return;
		$newMsgDelay = $newMsg['message_receiver']['auto']['usercreation']['delay'];

		if(empty($oldMsg->message_receiver['auto']['usercreation'])){

			$delay = strtotime($newMsgDelay['duration'].' '.$newMsgDelay['timevalue']) - time();

			$joomsmsquery = $this->_getJoomsmsQuery($oldMsg->message_id, $delay);
			$nbUser = $joomsmsquery->count($newMsg['message_receiver_table']);
			if(empty($nbUser)) return;

			$timeDisplayed = JOOMSMS::getDate(time() - $delay);

			$joomsmsquery = $this->_getJoomsmsQuery($oldMsg->message_id, '');
			$nbUserAll = $joomsmsquery->count($newMsg['message_receiver_table']);

			$text = $toggleHelper->toggleText('plgtrigger', $oldMsg->message_id.'_delay', '', '&function=onJoomSMSAutoMsgAdd&plg=joomsmsusercreation&plgtype=system&delay=1&msgId='.$oldMsg->message_id, JText::sprintf('SMS_ADD_NEW_MESSAGE_QUEUE_SUBSCRIBED_AFTER', $nbUser, $timeDisplayed));
			$text .= '<br />';
			$text .= $toggleHelper->toggleText('plgtrigger', $oldMsg->message_id, '', '&function=onJoomSMSAutoMsgAdd&plg=joomsmsusercreation&plgtype=system&msgId='.$oldMsg->message_id, JText::sprintf('SMS_ADD_NEW_MESSAGE_QUEUE_SUBSCRIBED_ALL', $nbUserAll));

			JOOMSMS::enqueueMessage($text, 'notice');
			return;
		}

		if(!isset($oldMsg->message_receiver['auto']['usercreation']['delay'])) return;
		$oldMsgDelay = $oldMsg->message_receiver['auto']['usercreation']['delay'];

		if(($oldMsgDelay['duration'] !== $newMsgDelay['duration']) || ($oldMsgDelay['timevalue'] !== $newMsgDelay['timevalue'])){
			$difference = strtotime($newMsgDelay['duration'].' '.$newMsgDelay['timevalue']) - strtotime($oldMsgDelay['duration'].' '.$oldMsgDelay['timevalue']);
			$text = JText::_('SMS_MESSAGE_CHANGED_DELAY_INFORMED');
			$text .= ' '.$toggleHelper->toggleText('plgtrigger', $oldMsg->message_id, '', '&function=onJoomSMSAutoMsgUpdate&plg=joomsmsusercreation&plgtype=system&difference='.$difference.'&msgId='.$oldMsg->message_id, JText::_('SMS_MESSAGE_CHANGED_DELAY'));

			JOOMSMS::enqueueMessage($text, 'notice');
		}
	}

	private function _getJoomsmsQuery($messageId, $msgDelay){
		if(!$this->init()) return;

		$messageClass = JOOMSMS::get('class.message');
		$message = $messageClass->get($messageId);
		$delayIsPresent = JRequest::getInt('delay', '');

		$integration = JOOMSMS::getIntegration($message->message_receiver_table);

		$joomsmsquery = JOOMSMS::get('class.joomsmsquery');
		$integration->initQuery($joomsmsquery);
		$joomsmsquery->addMessageFilters($message);
		if($delayIsPresent) $joomsmsquery->where[] = 'UNIX_TIMESTAMP(joomusers.`registerDate`) >'.(time() - $msgDelay);
		return $joomsmsquery;
	}

	public function ajax_onJoomSMSAutoMsgUpdate(){
		if(!$this->init()) return;

		$messageId = JRequest::getInt('msgId', '');
		if(empty($messageId)) return;

		$diff = JRequest::getInt('difference', '');
		if(empty($diff)) return;

		$queueClass = JOOMSMS::get('class.queue');
		$queueClass->plgQueueUpdateSenddate($messageId, $diff);
	}

	public function ajax_onJoomSMSAutoMsgAdd(){
		if(!$this->init()) return;

		$messageId = JRequest::getInt('msgId', '');
		if(empty($messageId)) return;

		$db = JFactory::getDBO();
		$messageClass = JOOMSMS::get('class.message');
		$newMessage = $messageClass->get($messageId);
		if(empty($newMessage->message_id)){
			echo 'Could not load messageId '.$messageId;
			exit;
		}

		$senddate = strtotime($newMessage->message_receiver['auto']['usercreation']['delay']['duration'].' '.$newMessage->message_receiver['auto']['usercreation']['delay']['timevalue']) - time();

		$integration = JOOMSMS::getIntegration($newMessage->message_receiver_table);
		$joomsmsquery = $this->_getJoomsmsQuery($messageId, $senddate);

		$querySelect = $joomsmsquery->getQuery(array($newMessage->message_id.','.$integration->tableAlias.'.'.$integration->primaryField.','.$db->Quote($newMessage->message_receiver_table).', UNIX_TIMESTAMP(joomusers.`registerDate`) +'.$senddate.',0,2'));
		$finalQuery = 'INSERT IGNORE INTO `#__joomsms_queue` (`queue_message_id`,`queue_receiver_id`,`queue_receiver_table`,`queue_senddate`,`queue_try`,`queue_priority`) '.$querySelect;
		$db->setQuery($finalQuery);
		$db->query();
		$nbinserted = $db->getAffectedRows();

		echo JText::sprintf('SMS_ADDED_QUEUE', $nbinserted);
		exit;
	}



	function _createCustomer($user){
		if(!$this->init()) return;

		$config = JOOMSMS::config();
		$db = JFactory::getDBO();

		$defaultCreditsURL = $config->get('default_credits_url', '');

		$senderProfileClass = JOOMSMS::get('class.senderprofile');
		$defaultSenderProfileId = $senderProfileClass->getDefaultSenderProfileId();

		$query = 'INSERT INTO #__joomsms_customer (customer_joomid, customer_senderprofile_id, customer_credits, customer_credits_url) VALUES ('.intval($user['id']).', '.intval($defaultSenderProfileId).', 0, '.$db->quote($defaultCreditsURL).')';

		$db->setQuery($query);
		$db->query();
	}
}//endclass
